@@ -31,7 +31,6 @@ def _executable(name):
 
 # GAMS
 GAMS_EXECUTABLE = _executable("gams")
-GAMSIDE_EXECUTABLE = _executable("gamside")
 
 # Julia
 JULIA_EXECUTABLE = _executable("julia")
@@ -46,6 +45,3 @@ EMBEDDED_PYTHON = os.path.join(APPLICATION_PATH, "tools", "python.exe")
 
 # Tool output directory name
 TOOL_OUTPUT_DIR = "output"
-
-# Gimlet default work directory name
-GIMLET_WORK_DIR_NAME = "work"
