@@ -37,6 +37,10 @@ if sys.platform == "win32":
     from subprocess import CREATE_NEW_PROCESS_GROUP, CREATE_NO_WINDOW
 
 
+class PersistentIsDead(Exception):
+    pass
+
+
 class PersistentManagerBase:
     def __init__(self, args, group_id):
         """
@@ -158,12 +162,20 @@ class PersistentManagerBase:
         cmd = os.linesep.join(lines)
         if len(lines) == 1:
             cmd += os.linesep
-        result = self._communicate("is_complete", cmd)
+        try:
+            result = self._communicate("is_complete", cmd)
+        except PersistentIsDead:
+            return None
         if result.strip() != "true":
             return None
         return cmd
 
     def drain_queue(self):
+        """Empties the message queue.
+
+        Yields:
+            dict: messages still in the queue
+        """
         while True:
             try:
                 yield self._msg_queue.get(timeout=0.02)
@@ -181,6 +193,7 @@ class PersistentManagerBase:
         Args:
             cmd (str)
             add_history (bool)
+            catch_exception (bool)
 
         Yields:
             dict: message
@@ -203,13 +216,17 @@ class PersistentManagerBase:
         Args:
             cmd (str): Command to pass to the persistent process
             add_history (bool): Whether to add the command to history
+            catch_exception (bool): whether to catch and report exceptions in the REPL as errors
         """
         with self._lock:
             self._msg_queue.put({"type": "stdin", "data": cmd})
             self.command_successful = self._issue_command(cmd, catch_exception=catch_exception)
             if self.command_successful:
                 if add_history:
-                    self._communicate("add_history", cmd, receive=False)
+                    try:
+                        self._communicate("add_history", cmd, receive=False)
+                    except PersistentIsDead:
+                        return
                 self.command_successful &= self._wait()
         self._msg_queue.put({"type": "command_finished"})
 
@@ -218,6 +235,7 @@ class PersistentManagerBase:
 
         Args:
             cmd (str): Command to pass to the persistent process
+            catch_exception (bool):
 
         Returns:
             bool: True if command was issued successfully, False otherwise
@@ -225,7 +243,10 @@ class PersistentManagerBase:
         if catch_exception:
             cmd = self._catch_exception_command(cmd) if cmd else ""
         cmd += os.linesep
-        self._persistent.stdin.write(cmd.encode("UTF8"))
+        try:
+            self._persistent.stdin.write(cmd.encode("UTF8"))
+        except ValueError:
+            return False
         try:
             self._persistent.stdin.flush()
             return True
@@ -250,7 +271,9 @@ class PersistentManagerBase:
         host, port = queue.get()  # This blocks until the server is listening
         ping = self._ping_command(host, port)
         if not self._issue_command(ping, catch_exception=False):
+            # Ping failed because pipe is broken, i.e. kernel died
             thread.join()
+            self._msg_queue.put({"type": "stdout", "data": "Kernel died (×_×)"})
             return False
         result = queue.get()  # This blocks until the server is done
         thread.join()
@@ -307,6 +330,8 @@ class PersistentManagerBase:
         Returns:
             str or NoneType: response, or None if the ``receive`` argument is False
         """
+        if not self.is_persistent_alive():
+            raise PersistentIsDead()
         req_args_sep = '\u001f'  # Unit separator
         args_sep = '\u0091'  # Private Use 1
         args = args_sep.join(args)
@@ -332,7 +357,10 @@ class PersistentManagerBase:
         Returns:
             list(str): List of options
         """
-        result = self._communicate("completions", text)
+        try:
+            result = self._communicate("completions", text)
+        except PersistentIsDead:
+            return []
         if result is None:
             return []
         return result.strip().split(" ")
@@ -349,10 +377,21 @@ class PersistentManagerBase:
             str
         """
         sense = "backwards" if backwards else "forward"
-        return self._communicate("history_item", text, prefix, sense).strip()
+        try:
+            return self._communicate("history_item", text, prefix, sense).strip()
+        except PersistentIsDead:
+            return ""
 
     def restart_persistent(self):
-        """Restarts the persistent process."""
+        """Restarts the persistent process.
+
+        Yields:
+            dict: messages still in the queue
+        """
+        if not self.is_persistent_alive():
+            self._start_persistent()
+            self._wait()
+            return
         self._persistent.kill()
         self._persistent.wait()
         self._persistent.stdout = os.devnull
@@ -577,6 +616,17 @@ class _PersistentManagerFactory(metaclass=Singleton):
             return
         pm.interrupt_persistent()
 
+    def kill_persistent(self, key):
+        """Kills a persistent process.
+
+        Args:
+            key (tuple): persistent identifier
+        """
+        pm = self.persistent_managers.get(key)
+        if pm is None:
+            return
+        pm.kill_process()
+
     def issue_persistent_command(self, key, cmd):
         """Issues a command to a persistent process.
 
@@ -672,6 +722,11 @@ def interrupt_persistent(key):
     _persistent_manager_factory.interrupt_persistent(key)
 
 
+def kill_persistent(key):
+    """See _PersistentManagerFactory."""
+    _persistent_manager_factory.kill_persistent(key)
+
+
 def kill_persistent_processes():
     """Kills all persistent processes.
 
@@ -714,15 +769,15 @@ def get_persistent_history_item(key, text, prefix, backwards):
 class PersistentExecutionManagerBase(ExecutionManagerBase):
     """Base class for managing execution of commands on a persistent process."""
 
-    def __init__(self, logger, args, commands, alias, group_id=None):
+    def __init__(self, logger, args, commands, alias, kill_completed_processes, group_id=None):
         """
         Args:
             logger (LoggerInterface): a logger instance
             args (list): List of args to start the persistent process
             commands (list): List of commands to execute in the persistent process
             alias (str): an alias name for the manager
+            kill_completed_processes (bool): if True, the persistent process will be killed after execution
             group_id (str, optional): item group that will execute using this kernel
-            server_ip (str): Engine Server IP address
         """
         super().__init__(logger)
         self._args = args
@@ -731,6 +786,7 @@ class PersistentExecutionManagerBase(ExecutionManagerBase):
         self._persistent_manager = _persistent_manager_factory.new_persistent_manager(
             self.persistent_manager_factory, logger, args, group_id
         )
+        self._kill_completed = kill_completed_processes
 
     @property
     def alias(self):
@@ -766,11 +822,17 @@ class PersistentExecutionManagerBase(ExecutionManagerBase):
             return 0
         finally:
             self._persistent_manager.set_running_until_completion(False)
+            if self._kill_completed:
+                self._persistent_manager.kill_process()
+                self._logger.msg_persistent_execution.emit({"type": "persistent_killed"})
 
     def stop_execution(self):
         """See base class."""
         if self._persistent_manager is not None:
             self._persistent_manager.interrupt_persistent()
+            if self._kill_completed:
+                self._persistent_manager.kill_process()
+                self._logger.msg_persistent_execution.emit({"type": "persistent_killed"})
 
 
 class JuliaPersistentExecutionManager(PersistentExecutionManagerBase):
