@@ -43,9 +43,10 @@ class ProjectItemResource:
             provider_name (str): The name of the item that provides the resource
             type_ (str): The resource type, currently available types:
 
-                - "file": url points to the file's path
+                - "file": url points to a local file
                 - "file_pack": resource is part of a pack; url points to the file's path
                 - "database": url is the databases url
+                - "url": url is a generic URL
             label (str): A label that identifies the resource.
             url (str, optional): The url of the resource.
             metadata (dict): Additional metadata providing extra information about the resource.
@@ -74,8 +75,9 @@ class ProjectItemResource:
                 "current": self.metadata.get("current"),
                 "precursors": self.metadata.get("precursors", set()),
             }
+            db_server_manager_queue = self.metadata["db_server_manager_queue"]
             with closing_spine_db_server(
-                self.url, memory=self.metadata.get("memory", False), ordering=ordering
+                db_server_manager_queue, self.url, memory=self.metadata.get("memory", False), ordering=ordering
             ) as server_url:
                 if db_checkin:
                     SpineDBClient.from_server_url(server_url).db_checkin()
@@ -84,19 +86,22 @@ class ProjectItemResource:
                 finally:
                     if db_checkout:
                         SpineDBClient.from_server_url(server_url).db_checkout()
+        elif self.type_ == "url":
+            yield self.url
         else:
             yield self.path if self.hasfilepath else ""
 
     def quick_db_checkout(self):
         if self.type_ != "database":
             return
+        db_server_manager_queue = self.metadata["db_server_manager_queue"]
         ordering = {
             "id": self._identifier,
             "part_count": self.metadata.get("part_count", PartCount()),
             "current": self.metadata.get("current"),
             "precursors": self.metadata.get("precursors", set()),
         }
-        quick_db_checkout(ordering)
+        quick_db_checkout(db_server_manager_queue, ordering)
 
     def clone(self, additional_metadata=None):
         """Clones this resource and optionally updates the clone's metadata.
@@ -218,7 +223,7 @@ class LabelArg(CmdLineArg):
 
 def database_resource(provider_name, url, label=None, filterable=False):
     """
-    Constructs a database resource.
+    Constructs a Spine database resource.
 
     Args:
         provider_name (str): resource provider's name
@@ -231,6 +236,18 @@ def database_resource(provider_name, url, label=None, filterable=False):
     return ProjectItemResource(provider_name, "database", label, url, filterable=filterable)
 
 
+def url_resource(provider_name, url, label):
+    """
+    Constructs a generic URL resource.
+
+    Args:
+        provider_name (str): resource provider's name
+        url (str): database URL
+        label (str): resource label
+    """
+    return ProjectItemResource(provider_name, "url", label, url)
+
+
 def file_resource(provider_name, file_path, label=None):
     """
     Constructs a file resource.
@@ -314,8 +331,9 @@ def get_labelled_sources(resources):
         dict: a mapping from resource label to list of URLs or file paths
     """
     d = {}
+    url_resources = ("database", "url")
     for resource in resources:
-        if resource.type_ == "database":
+        if resource.type_ in url_resources:
             d.setdefault(resource.label, []).append(resource.url)
         elif resource.hasfilepath:
             d.setdefault(resource.label, []).append(resource.path)
@@ -339,22 +357,25 @@ def make_cmd_line_arg(arg_spec):
 
 
 def labelled_resource_args(resources, stack, db_checkin=False, db_checkout=False):
-    """
+    """Generates command line arguments for each resource.
+
     Args:
         resources (Iterable of ProjectItemResource): resources to process
-        stack (ExitStack)
+        stack (ExitStack): context manager to ensure resources get closed properly
+        db_checkin (bool): is database checkin required
+        db_checkout (bool): is database checkout required
 
     Yields:
-        dict: mapping from resource label to resource args.
+        dict: mapping from resource label to a list of resource args.
     """
     result = {}
     single_resources, pack_resources = extract_packs(resources)
     for resource in single_resources:
-        result[resource.label] = stack.enter_context(resource.open(db_checkin=db_checkin, db_checkout=db_checkout))
+        result[resource.label] = [stack.enter_context(resource.open(db_checkin=db_checkin, db_checkout=db_checkout))]
     for label, resources_ in pack_resources.items():
-        result[label] = " ".join(
+        result[label] = [
             stack.enter_context(r.open(db_checkin=db_checkin, db_checkout=db_checkout)) for r in resources_
-        )
+        ]
     return result
 
 
@@ -363,7 +384,7 @@ def expand_cmd_line_args(args, label_to_arg, logger):
 
     Args:
         args (list of CmdLineArg): command line arguments
-        label_to_arg (dict): a mapping from resource label to cmd line argument
+        label_to_arg (dict): a mapping from resource label to list of cmd line arguments
         logger (LoggerInterface): a logger
 
     Returns:
@@ -379,5 +400,5 @@ def expand_cmd_line_args(args, label_to_arg, logger):
             logger.msg_warning.emit(f"No resources matching argument '{arg}'.")
             continue
         if expanded:
-            expanded_args.append(expanded)
+            expanded_args.extend(expanded)
     return expanded_args
