@@ -21,8 +21,8 @@ import time
 import threading
 import ipaddress
 import enum
-import zmq
 import uuid
+import zmq
 from zmq.auth.thread import ThreadAuthenticator
 from spine_engine.server.util.server_message import ServerMessage
 from spine_engine.server.request import Request
@@ -31,6 +31,7 @@ from spine_engine.server.ping_service import PingService
 from spine_engine.server.project_extractor_service import ProjectExtractorService
 from spine_engine.server.project_retriever_service import ProjectRetrieverService
 from spine_engine.server.persistent_execution_service import PersistentExecutionService
+from spine_engine.server.project_remover_service import ProjectRemoverService
 
 
 class ServerSecurityModel(enum.Enum):
@@ -62,16 +63,16 @@ class EngineServer(threading.Thread):
             self.secret_keys_dir = os.path.join(base_dir, 'private_keys')
             if not os.path.exists(self.keys_dir):
                 raise ValueError(f"Security folder: {self.keys_dir} does not exist")
-            elif not os.path.exists(self.public_keys_dir):
+            if not os.path.exists(self.public_keys_dir):
                 raise ValueError(f"Security folder: {self.public_keys_dir} does not exist")
-            elif not os.path.exists(self.secret_keys_dir):
+            if not os.path.exists(self.secret_keys_dir):
                 raise ValueError(f"Security folder: {self.secret_keys_dir} does not exist")
             self._sec_folder = sec_folder
             self._sec_model_state = ServerSecurityModel.STONEHOUSE
         self.protocol = protocol
         self.port = port
         self.auth = None
-        # NOTE: Contexts are thread-safe! Sockets are NOT!. Do not use or close
+        # NOTE: Contexts are thread-safe! Sockets are NOT! Do not use or close
         # sockets except in the thread that created them.
         self._context = zmq.Context()
         self.ctrl_msg_sender = self._context.socket(zmq.PAIR)
@@ -178,6 +179,14 @@ class EngineServer(threading.Thread):
                             self.send_init_failed_reply(frontend, request.connection_id(), msg)
                             continue
                         worker = PersistentExecutionService(self._context, request, job_id, exec_mngr)
+                    elif request.cmd() == "remove_project":
+                        project_dir = project_dirs.get(request.request_id(), None)  # Get project dir based on job_id
+                        if not project_dir:
+                            print(f"Project for job_id:{request.request_id()} not found")
+                            msg = f"Project directory for job_id {request.request_id()} not found"
+                            self.send_init_failed_reply(frontend, request.connection_id(), msg)
+                            continue
+                        worker = ProjectRemoverService(self._context, request, job_id, project_dir)
                     else:
                         print(f"Unknown command {request.cmd()} requested")
                         msg = f"Server error: Unknown command '{request.cmd()}' requested"
