@@ -37,6 +37,7 @@ from dagster import (
     execute_pipeline_iterator,
 )
 from spinedb_api import append_filter_config, name_from_dict
+from spinedb_api.spine_db_server import db_server_manager
 from spinedb_api.filters.tools import filter_config
 from spinedb_api.filters.scenario_filter import scenario_name_from_dict
 from spinedb_api.filters.execution_filter import execution_filter_config
@@ -127,12 +128,15 @@ class SpineEngine:
         self._connections_by_destination = dict()
         node_successors = {item_name: list() for item_name in self._items}
         for connection in self._connections:
+            if not connection.ready_to_execute():
+                notifications = " ".join(connection.notifications())
+                raise EngineInitFailed(f"Link {connection.name} is not ready for execution. {notifications}")
             source, destination = connection.source, connection.destination
             self._connections_by_source.setdefault(source, list()).append(connection)
             self._connections_by_destination.setdefault(destination, list()).append(connection)
-            sucessors = node_successors.get(source)
-            if sucessors is not None:
-                sucessors.append(destination)
+            successors = node_successors.get(source)
+            if successors is not None:
+                successors.append(destination)
         self._settings = AppSettings(settings if settings is not None else {})
         _set_resource_limits(self._settings, SpineEngine._resource_limit_lock)
         enable_persistent_process_creation()
@@ -171,7 +175,10 @@ class SpineEngine:
         self._answered_prompts = {}
         self.resources_per_item = {}  # Tuples of (forward resources, backward resources) from last execution
         self._timestamp = create_timestamp()
+        self._db_server_manager_queue = None
+        self._thread = threading.Thread(target=self.run)
         self._event_stream = self._get_event_stream()
+        self._multiprocess_manager = mp.Manager()
 
     def _make_item_specifications(self, specifications, project_item_loader, items_module_name):
         """Instantiates item specifications.
@@ -231,19 +238,29 @@ class SpineEngine:
         Yields:
             tuple: event type and data
         """
-        threading.Thread(target=self.run).start()
+        self._thread.start()
         while True:
             msg = self._queue.get()
             yield msg
             if msg[0] == "dag_exec_finished":
                 break
+        self._thread.join()
 
     def answer_prompt(self, item_name, accepted):
         """Answers the prompt for the specified item, either accepting or rejecting it."""
         self._prompt_queues[item_name].put(accepted)
 
+    def wait(self):
+        if self._thread.is_alive():
+            self._thread.join()
+        self._multiprocess_manager.shutdown()
+
     def run(self):
         """Runs this engine."""
+        with db_server_manager() as self._db_server_manager_queue:
+            self._do_run()
+
+    def _do_run(self):
         self._state = SpineEngineState.RUNNING
         run_config = {
             "loggers": {"console": {"config": {"log_level": "CRITICAL"}}},
@@ -402,6 +419,8 @@ class SpineEngine:
             context.log.info(f"Item Name: {item_name}")
             item = self.make_item(item_name, ED.BACKWARD)
             resources = item.output_resources(ED.BACKWARD)
+            for r in resources:
+                r.metadata["db_server_manager_queue"] = self._db_server_manager_queue
             yield Output(value=resources, output_name=f"{ED.BACKWARD}_output")
 
         input_defs = []
@@ -496,13 +515,14 @@ class SpineEngine:
         resources_iterator = self._filtered_resources_iterator(
             item_name, forward_resource_stacks, backward_resources, self._timestamp
         )
+        item_lock = self._multiprocess_manager.Lock()
         for flt_fwd_resources, flt_bwd_resources, filter_id in resources_iterator:
             self.resources_per_item[item_name] = (flt_fwd_resources, flt_bwd_resources)
             item = self.make_item(item_name, ED.FORWARD)
             item.filter_id = filter_id
             thread = threading.Thread(
                 target=self._execute_item_filtered,
-                args=(item, flt_fwd_resources, flt_bwd_resources, output_resources_list, success),
+                args=(item, flt_fwd_resources, flt_bwd_resources, output_resources_list, item_lock, success),
             )
             threads.append(thread)
         for thread in threads:
@@ -518,7 +538,7 @@ class SpineEngine:
         return success[0], output_resources_list
 
     def _execute_item_filtered(
-        self, item, filtered_forward_resources, filtered_backward_resources, output_resources_list, success
+        self, item, filtered_forward_resources, filtered_backward_resources, output_resources_list, item_lock, success
     ):
         """Executes the given item using the given filtered resources. Target for threads in ``_execute_item``.
 
@@ -528,20 +548,22 @@ class SpineEngine:
             filtered_backward_resources (list(ProjectItemResource))
             output_resources_list (list(list(ProjectItemResource))): A list to append the output resources
                 generated by the item.
+            item_lock (mp.Lock): Shared lock for parallel executions.
             success (list): A list of one element, to write the outcome of the execution.
         """
         self._running_items.append(item)
         if self._execution_permits[self._solid_names[item.name]]:
-            item_finish_state = item.execute(filtered_forward_resources, filtered_backward_resources)
+            item_finish_state = item.execute(filtered_forward_resources, filtered_backward_resources, item_lock)
             item.finish_execution(item_finish_state)
         else:
-            item.exclude_execution(filtered_forward_resources, filtered_backward_resources)
+            item.exclude_execution(filtered_forward_resources, filtered_backward_resources, item_lock)
             item_finish_state = ItemExecutionFinishState.EXCLUDED
         filter_stack = sum((r.metadata.get("filter_stack", ()) for r in filtered_forward_resources), ())
         output_resources = item.output_resources(ED.FORWARD)
         for resource in output_resources:
             resource.metadata["filter_stack"] = filter_stack
             resource.metadata["filter_id"] = item.filter_id
+            resource.metadata["db_server_manager_queue"] = self._db_server_manager_queue
         output_resources_list.append(output_resources)
         success[0] = item_finish_state  # FIXME: We need a Lock here
         self._running_items.remove(item)
@@ -815,8 +837,10 @@ def validate_single_jump(jump, jumps, dag, items_by_jump=None):
         jump (Jump): the jump to check
         jumps (list of Jump): all jumps in dag
         dag (DiGraph): jumps' DAG
-        items_by_jump (dict): mapping jumps to a set of items in between destination and source
+        items_by_jump (dict, optional): mapping jumps to a set of items in between destination and source
     """
+    if not jump.ready_to_execute():
+        raise EngineInitFailed(f"Jump {jump.name} is not ready for execution.")
     if items_by_jump is None:
         items_by_jump = _get_items_by_jump(jumps, dag)
     for other in jumps:
@@ -827,7 +851,7 @@ def validate_single_jump(jump, jumps, dag, items_by_jump=None):
         jump_items = items_by_jump[jump]
         other_items = items_by_jump[other]
         intersection = jump_items & other_items
-        if intersection not in ({}, jump_items, other_items):
+        if intersection not in (set(), jump_items, other_items):
             raise EngineInitFailed(f"{jump.name} cannot partially overlap {other.name}.")
     if not dag.has_node(jump.destination):
         raise EngineInitFailed(f"Loop destination '{jump.destination}' not found in DAG")
