@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -17,6 +17,9 @@ The ReturningProcess class.
 """
 
 import multiprocessing as mp
+import threading
+from contextlib import contextmanager
+from enum import Enum, auto, unique
 from .execution_resources import one_shot_process_semaphore
 
 
@@ -25,6 +28,7 @@ class ReturningProcess(mp.Process):
         super().__init__(*args, **kwargs)
         self._queue = mp.Queue()
         self._terminated = False
+        self.maybe_idle = _MaybeIdle()  # target functions should enter this context whenever they may become idle
 
     def run_until_complete(self):
         """Starts the process and joins it after it has finished.
@@ -35,17 +39,18 @@ class ReturningProcess(mp.Process):
         with one_shot_process_semaphore:
             if self._terminated:
                 return (False,)
-            self.start()
-            return_value = self._queue.get()
-            self.join()
+            with self.maybe_idle.listen():
+                self.start()
+                return_value = self._queue.get()
+                self.join()
             return return_value
 
     def run(self):
         if not self._target:
             self._queue.put((False,))
             return
-        result = self._target(*self._args, **self._kwargs)
-        self._queue.put(result)
+        return_value = self._target(self, *self._args, **self._kwargs)
+        self._queue.put(return_value)
 
     def terminate(self):
         self._terminated = True
@@ -54,6 +59,48 @@ class ReturningProcess(mp.Process):
         self._queue.put((False,))
 
 
+class _MaybeIdle:
+    """A context manager to temporarily release the one shot process semaphore.
+    This allows us to prevent deadlocks when processes need to wait for others to do certain actions in order to
+    proceed (e.g., when executable items need to write to a DB in a certain order and we can't predict in which order
+    their processes will start).
+    """
+
+    @unique
+    class _Event(Enum):
+        ENTER = auto()
+        EXIT = auto()
+
+    def __init__(self):
+        self._queue = mp.Queue()
+
+    def __enter__(self):
+        self._queue.put(self._Event.ENTER)
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._queue.put(self._Event.EXIT)
+
+    @contextmanager
+    def listen(self):
+        thread = threading.Thread(target=self._do_listen)
+        thread.start()
+        try:
+            yield
+        finally:
+            self._queue.put(None)
+            thread.join()
+
+    def _do_listen(self):
+        while True:
+            event = self._queue.get()
+            if event == self._Event.ENTER:
+                one_shot_process_semaphore.release()
+            elif event == self._Event.EXIT:
+                one_shot_process_semaphore.acquire()
+            elif event is None:
+                break
+
+
 if __name__ == "__main__":
     # https://docs.python.org/3.7/library/multiprocessing.html?highlight=freeze_support#multiprocessing.freeze_support
     mp.freeze_support()
